;; Copyright 2006, 2007, 2008, 2009, 2010, 2013, 2014, 2019 Kevin Ryde.

;; This file is part of aumix-mode.
;;
;; aumix-mode is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; aumix-mode is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with aumix-mode; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  ;; (load (expand-file-name "../debian/emacsen-startup"))

  (let ((count   0)
        (tempbuf "*t-aumix-mode--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    ;; (mapcar (lambda (func)
    ;;           (unless (fboundp func)
    ;;             (error "`%S' not autoloaded" func))
    ;;           (setq count (1+ count)))
    ;;         '(aumix))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'aumix-mode)

    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'aumix-mode)
      (kill-buffer nil))
    (setq count (1+ count))

    (if (not (fboundp 'custom-group-of-mode))
        (message "skip, no `custom-group-of-mode' in this emacs")
      (let* ((mode 'aumix-mode)
             (want 'aumix-mode)
             (got  (custom-group-of-mode mode)))
        (unless (equal got want)
          (error "custom-group-of-mode is %S expected %S" got want)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; risky-local-variable-p

    (if (not (fboundp 'risky-local-variable-p))
        (message "skip, no `risky-local-variable-p'")

      (mapcar (lambda (symbol)
                (unless (boundp symbol)
                  (error "oops, no variable: %S" symbol))
                (unless (risky-local-variable-p symbol (symbol-value symbol))
                  (error "%S not `risky-local-variable-p'" symbol))
                (setq count (1+ count)))
              '(aumix-mode-program
                aumix-mode-switches
                aumix-mode-hook)))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (unload-feature 'aumix-mode t)

    ;;-------------------------------------------------------------------------
    ;; desktop.el

    (if (file-exists-p ".emacs.desktop")
        (delete-file ".emacs.desktop"))
    (if (file-exists-p ".emacs.desktop.lock")
        (delete-file ".emacs.desktop.lock"))

    (require 'aumix-mode)
    (aumix)
    (require 'desktop)
    (if (or (not (boundp 'desktop-file-version))
            (< (string-to-number desktop-file-version) 206))
        (message "skip, desktop.el before version 206")

      (desktop-save "." t)

      (progn
        (let ((noninteractive nil))
          (desktop-read ".")
          (desktop-release-lock "."))
        (unless (get-buffer "*aumix*")
          (error "desktop-read didn't leave existing *aumix* buffer"))
        (setq count (1+ count)))

      (progn
        (kill-buffer "*aumix*")
        (let ((noninteractive nil))
          (desktop-read ".")
          (desktop-release-lock "."))

        (unless (get-buffer "*aumix*")
          (error "desktop-read didn't re-create *aumix* buffer\nbuffers:\n%S\nhandlers:\n%S"
                 (mapcar 'buffer-name (buffer-list))
                 desktop-buffer-mode-handlers))
        (setq count (1+ count)))

      (progn
        (set-buffer "*aumix*")
        (rename-buffer "*aumix-testing*")
        (desktop-save "." t)
        (kill-buffer "*aumix-testing*")

        (let ((noninteractive nil))
          (desktop-read ".")
          (desktop-release-lock "."))

        (if (get-buffer "*aumix-testing*")
            (kill-buffer "*aumix-testing*")
          (error "desktop-read didn't re-create *aumix-testing* buffer"))
        (setq count (1+ count))))

    (if (file-exists-p ".emacs.desktop")
        (delete-file ".emacs.desktop"))
    (if (file-exists-p ".emacs.desktop.lock")
        (delete-file ".emacs.desktop.lock"))

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
